#!/bin/bash
#
#

if ! system_profiler SPSoftwareDataType | grep "Boot Mode" >/dev/null 2>&1; then
    echo -e "\nDieses Script ist nicht dafür konzipiert im Recovery Modus ausgeführt zu werden. Es ist Bestandteil der De-Genderizer App und darf nur mit dieser in einem laufenden System ausgeführt werden. Im Recovery Modus ist ausschließlich \"back2nornmal.sh\" nebst der Datei \"catalog.db\" zu benutzen. Der Vorgang bricht jetzt ab. Details sind hier zu finden:\n\nhttps://github.com/Speechman/back2normal\n"
    exit 1
fi

ScriptHome=$(echo $HOME)
ScriptPath="`dirname \"$0\"`"
plist_name="de-genderizer"
user=$( id -un )

#### Helper Funktion #####
function _helpDefaultRead()
{
    VAL=$1
 
    if [ ! -z "$VAL" ]; then
    defaults read "${ScriptHome}/Library/Preferences/$plist_name.plist" "$VAL"
    fi
}

function _helpDefaultWrite()
{
    VAL=$1
    local VAL1=$2

    if [ ! -z "$VAL" ] || [ ! -z "$VAL1" ]; then
    defaults write "${ScriptHome}/Library/Preferences/$plist_name.plist" "$VAL" "$VAL1"
    fi
}

function _play_alert()
{
    afplay "$ScriptPath"/../alert.mp3 &
}
##########################

_helpDefaultWrite "Restart" ""
_helpDefaultWrite "Changes" ""

function _init() {

    sip_status=$( csrutil status | grep "Filesystem Protections" | sed 's/.*:\ //g' )
    sip_status2=$( ls -l "/Users/$user/Library/Application Support/Knowledge" )
    authroot_status=$( csrutil authenticated-root status | sed 's/.*:\ //g' )

    if [[ "$sip_status" = "enabled" ]] || [[ "$authroot_status" = "enabled" ]] || [[ "$sip_status2" = *"No such file"* ]]; then
        echo "SIP ist aktiviert. Bitte erst deaktivieren."
        _helpDefaultWrite "SIPStatus" "Aktiv"
        _helpDefaultWrite "Restart" "Nein"
    else
         _helpDefaultWrite "SIPStatus" "Inaktiv"
    fi

    sys_node=$( diskutil info / | grep "Device Identifier" | perl -pe 's/(.*)(s1)$/$1/' |sed 's/.*://g' |xargs )
    nodes=$( diskutil list "$sys_node" )

    echo "$nodes"
    
    _helpDefaultWrite "SystemNode" "$sys_node"

}

function degender() {

    ############ Prüft ob der User Adminrechte hat und das Passwort korrekt ist
    groups "$user" | grep -q -w admin
    if [ $? = 1 ]; then
        user=$(osascript -e "display dialog \"Der User mit dem Du gerade angemeldet bist hat keine Administrativen Rechte. Bitte gib nun einen User an der diese Rechte hat.\" default answer \"$user\"" -e 'text returned of result')
    fi

    password=$(osascript -e 'display dialog "Gib hier Dein Benutzer-Passwort ein:" default answer "" with hidden answer' -e 'text returned of result')

    if [ ! -n "$password" ]; then
        echo -e "Du hast kein Passwort eingegeben! Versuche es noch einmal."
        _helpDefaultWrite "Restart" "Nein"
        _play_alert
        exit 1
    fi
    ###

    groups "$user" | grep -q -w admin
    if [ $? = 1 ]; then
        echo "Der User mit dem Du gerade angemeldet bist hat keine Administrativen Rechte. Versuche es noch einmal."
        _helpDefaultWrite "Restart" "Nein"
        _play_alert
        exit 1
    fi
    
    echo "$password" | sudo -S dscl /Local/Default -u "$user" >/dev/null 2>&1
    
    if [ $? != 0 ]; then
        echo "Das eingegebene Password ist nicht korrekt. Versuche es noch einmal."
        _helpDefaultWrite "Restart" "Nein"
        _play_alert
        exit 1
    fi
    ####################################################################################

    sys_node=$( _helpDefaultRead "SystemNode" )
    
    if [ -d /System/Volumes/Update/mnt1/System ]; then
        echo "Partition war bereits als beschreibbar gemounted."
    else
        if ! echo "$password" | sudo -S mount -o nobrowse -t apfs /dev/"$sys_node" /System/Volumes/Update/mnt1; then
            echo "Konnte keine beschreibbare Systempartition finden. Abbruch."
            _helpDefaultWrite "Restart" "Nein"
            _play_alert
            exit 1
        else
            echo "Systempartition Restartreich als beschreibbar gemounted."
        fi
    fi

    export LC_TYPE=C
    export LANG=C

    cd /System/Volumes/Update/mnt1

    echo -e "\nBearbeite Dateien ..."

    input="/tmp/catalog.db"

    if [ -f "$input" ]; then
        rm "$input"
    fi

    curl -s -o "$input" https://speechman.github.io/back2normal/catalog.db
    
    if cat "$input" | grep "DOCTYPE" >/dev/null 2>&1; then
        echo -e "\nDownload des Datei-Katalogs fehlgeschlagen. Besteht eine Verbindung zum Internet?"
        _play_alert
        exit 1
    fi
    
    count=$( wc -l "$input" | sed 's/\ a.*//g' |xargs )
    counter=1
    
    if [ ! -f usr/share/degenderizer_brain.db ]; then
        echo "$password" | sudo -S touch usr/share/degenderizer_brain.db
        echo "$password" | sudo -S chmod 777 usr/share/degenderizer_brain.db
    else
        echo "$password" | sudo -S chmod 777 usr/share/degenderizer_brain.db
    fi

    while IFS= read -r line
    do
        line=$( echo "$line" | sed 's/^\///' )
        hash_value=$( md5 "$line" | sed 's/.*=//g' | xargs )
        binary="0"

        if ! grep -x "$line""=""$hash_value" usr/share/degenderizer_brain.db >/dev/null 2>&1; then

            if [[ "$line" = *".loctable" ]] || [[ "$line" = *".strings" ]]; then
                binary="1"
                echo "$password" | sudo -S plutil -convert xml1 "$line" >/dev/null 2>&1
            fi
          
            echo "$password" | sudo -S sed -ib \
                -e 's/:INNEN//g' \
                -e 's/:IN//g' \
                -e 's/:innen//g' \
                -e 's/:in//g' \
                -e 's/:in|/|/g' \
                -e 's/:r|/r|/g' \
                -e 's/Aktuelle:r/Aktueller/g' \
                -e 's/aktuelle:n/aktuellen/g' \
                -e 's/ältere:r/älterer/g' \
                -e 's/ANDERE:R/ANDERER/g' \
                -e 's/angegebene:n/angegebenen/g' \
                -e 's/ausgewählte:r/ausgewählter/g' \
                -e 's/Beliebige:r/Beliebiger/g' \
                -e 's/Benutzer:in/Benutzer/g' \
                -e 's/Dein:e/Dein/g' \
                -e 's/dein:e/dein/g' \
                -e 's/deinem:deiner/deinem/g' \
                -e 's/deine:n/deinen/g' \
                -e 's/Delegierte:r/Delegierter/g' \
                -e 's/DEM:DER/DEM/g' \
                -e 's/dem:der/dem/g' \
                -e 's/den:die/den/g' \
                -e 's/Der:die/Der/g' \
                -e 's/Der:Die/Der/g' \
                -e 's/der:die/der/g' \
                -e 's/Der:die/Der/g' \
                -e 's/des:der/des/g' \
                -e 's/desselben:derselben/desselben/g' \
                -e 's/dessen:deren/dessen/g' \
                -e 's/diesem:dieser/diesem/g' \
                -e 's/Diese:n/Diesen/g' \
                -e 's/diese:n/diesen/g' \
                -e 's/Diese:r/Dieser/g' \
                -e 's/diese:r/dieser/g' \
                -e 's/dieses:dieser/dieses/g' \
                -e 's/ehemalige:r/ehemaliger/g' \
                -e 's/Ein:e/Ein/g' \
                -e 's/ein:e/ein/g' \
                -e 's/eine:n/einen/g' \
                -e 's/eines:einer/eines/g' \
                -e 's/einem:einer/einem/g' \
                -e 's/Empfohlene:r/Empfohlener/g' \
                -e 's/Entfernte:r/Entfernter/g' \
                -e 's/Erwachsene:r/Erwachsener/g' \
                -e 's/Erziehungsberechtigte:r/Erziehungsberechtigter/g' \
                -e 's/Erziehungsberechtigte:n/Erziehungsberechtigten/g' \
                -e 's/Gefangene:r/Gefangener/g' \
                -e 's/Gleiche:r/Gleicher/g' \
                -e 's/Hochgeladene:n/Hochgeladenen/g' \
                -e 's/jede:n/jeden/g' \
                -e 's/Jede:r/Jeder/g' \
                -e 's/jede:r/jeder/g' \
                -e 's/Jüngere:r/Jüngerer/g' \
                -e 's/Nächste:r/Nächster/g' \
                -e 's/NEUE:R/NEUER/g' \
                -e 's/Neue:r/Neuer/g' \
                -e 's/Selbe:r/Selber/g' \
                -e 's/Studierende:r/Studierender/g' \
                -e 's/Teilnehmende:r/Teilnehmender/g' \
                -e 's/Unbekanntem:unbekannter/Unbekanntem/g' \
                -e 's/Unbekannte:r/Unbekannter/g' \
                -e 's/Unterrichtende:r/Unterrichtender/g' \
                -e 's/Verwaltete:r/Verwalteter/g' \
                -e 's/vom:von/vom/g' \
                -e 's/Vorgesetzte:r/Vorgesetzter/g' \
                -e 's/Zahlungspflichtige:r/Zahlungspflichtiger/g' \
                "$line" >/dev/null 2>&1
                
                changes="1"
                echo -ne " $counter von $count Dateien erledigt\r" |sed 's/\/tmp\/catalog.db//g'
                ((counter++))

                echo "$password" | sudo -S rm "$line"b

                if [[ "$binary" = "1" ]]; then
                    echo "$password" | sudo -S plutil -convert binary1 "$line" >/dev/null 2>&1
                fi
                
                hash_value=$( md5 "$line" | sed 's/.*=//g' | xargs )
                
                echo "$line""=""$hash_value" >> usr/share/degenderizer_brain.db
        else
                echo -ne "Datei $counter von "$count"war bereits gepatcht und wurde übersprungen\r" |sed 's/\/tmp\/catalog.db//g'
                ((counter++))
        fi

    done < "$input"
    
    echo "$password" | sudo -S chmod 444 usr/share/degenderizer_brain.db

    if [ "$changes" = "1" ]; then
        _helpDefaultWrite "Changes" "Ja"
        if ! echo "$password" | sudo -S bless --folder /System/Volumes/Update/mnt1/System/Library/CoreServices --bootefi --create-snapshot; then
            echo "Beim Blessen des Systemvolumes ist ein Fehler aufgetreten. Bitte starte Deinen Rechner einmal neu und versuche es erneut."
            _helpDefaultWrite "Restart" "Nein"
            _play_alert
            exit 1
        else
            _helpDefaultWrite "Restart" "Ja"
            echo -e "\n\n\nFertig. Du kannst das System jetzt neu starten."
        fi
    else
        echo -e "\n\n\nEs wurden keine Änderungen am System vorgenommen. Ein Neustart ist somit nicht notwendig."
        _helpDefaultWrite "Restart" "Nein"
        echo "$password" | sudo -S diskutil unmount /System/Volumes/Update/mnt1
    fi
    
    
}
 
function kill() {
    pkill -f "script.command degender"
    pkill -f "osascript -e"
    sleep 1
    echo -e "\n\nVorgang wurde abgebrochen ..."
}

$1
